"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicrosoftOneDrive = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const GenericFunctions_1 = require("./GenericFunctions");
const FileDescription_1 = require("./FileDescription");
const FolderDescription_1 = require("./FolderDescription");
class MicrosoftOneDrive {
    constructor() {
        this.description = {
            displayName: 'Microsoft OneDrive',
            name: 'microsoftOneDrive',
            icon: 'file:oneDrive.svg',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume Microsoft OneDrive API',
            defaults: {
                name: 'Microsoft OneDrive',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'microsoftOneDriveOAuth2Api',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    noDataExpression: true,
                    options: [
                        {
                            name: 'File',
                            value: 'file',
                        },
                        {
                            name: 'Folder',
                            value: 'folder',
                        },
                    ],
                    default: 'file',
                },
                ...FileDescription_1.fileOperations,
                ...FileDescription_1.fileFields,
                ...FolderDescription_1.folderOperations,
                ...FolderDescription_1.folderFields,
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < length; i++) {
            try {
                if (resource === 'file') {
                    if (operation === 'copy') {
                        const fileId = this.getNodeParameter('fileId', i);
                        const additionalFields = this.getNodeParameter('additionalFields', i);
                        const parentReference = this.getNodeParameter('parentReference', i);
                        const body = {};
                        if (parentReference) {
                            body.parentReference = { ...parentReference };
                        }
                        if (additionalFields.name) {
                            body.name = additionalFields.name;
                        }
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'POST', `/drive/items/${fileId}/copy`, body, {}, undefined, {}, { json: true, resolveWithFullResponse: true });
                        responseData = { location: responseData.headers.location };
                    }
                    if (operation === 'delete') {
                        const fileId = this.getNodeParameter('fileId', i);
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'DELETE', `/drive/items/${fileId}`);
                        responseData = { success: true };
                    }
                    if (operation === 'download') {
                        const fileId = this.getNodeParameter('fileId', i);
                        const dataPropertyNameDownload = this.getNodeParameter('binaryPropertyName', i);
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'GET', `/drive/items/${fileId}`);
                        const fileName = responseData.name;
                        if (responseData.file === undefined) {
                            throw new n8n_workflow_1.NodeApiError(this.getNode(), responseData, {
                                message: 'The ID you provided does not belong to a file.',
                            });
                        }
                        let mimeType;
                        if (responseData.file.mimeType) {
                            mimeType = responseData.file.mimeType;
                        }
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'GET', `/drive/items/${fileId}/content`, {}, {}, undefined, {}, { encoding: null, resolveWithFullResponse: true });
                        const newItem = {
                            json: items[i].json,
                            binary: {},
                        };
                        if (mimeType === undefined && responseData.headers['content-type']) {
                            mimeType = responseData.headers['content-type'];
                        }
                        if (items[i].binary !== undefined) {
                            Object.assign(newItem.binary, items[i].binary);
                        }
                        items[i] = newItem;
                        const data = Buffer.from(responseData.body);
                        items[i].binary[dataPropertyNameDownload] = await this.helpers.prepareBinaryData(data, fileName, mimeType);
                    }
                    if (operation === 'get') {
                        const fileId = this.getNodeParameter('fileId', i);
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'GET', `/drive/items/${fileId}`);
                    }
                    if (operation === 'search') {
                        const query = this.getNodeParameter('query', i);
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/drive/root/search(q='${query}')`);
                        responseData = responseData.filter((item) => item.file);
                    }
                    if (operation === 'share') {
                        const fileId = this.getNodeParameter('fileId', i);
                        const type = this.getNodeParameter('type', i);
                        const scope = this.getNodeParameter('scope', i);
                        const body = {
                            type,
                            scope,
                        };
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'POST', `/drive/items/${fileId}/createLink`, body);
                    }
                    if (operation === 'upload') {
                        const parentId = this.getNodeParameter('parentId', i);
                        const isBinaryData = this.getNodeParameter('binaryData', i);
                        const fileName = this.getNodeParameter('fileName', i);
                        if (isBinaryData) {
                            const binaryPropertyName = this.getNodeParameter('binaryPropertyName', 0);
                            const binaryData = this.helpers.assertBinaryData(i, binaryPropertyName);
                            const body = await this.helpers.getBinaryDataBuffer(i, binaryPropertyName);
                            let encodedFilename;
                            if (fileName !== '') {
                                encodedFilename = encodeURIComponent(fileName);
                            }
                            if (binaryData.fileName !== undefined) {
                                encodedFilename = encodeURIComponent(binaryData.fileName);
                            }
                            responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'PUT', `/drive/items/${parentId}:/${encodedFilename}:/content`, body, {}, undefined, { 'Content-Type': binaryData.mimeType, 'Content-length': body.length }, {});
                            responseData = JSON.parse(responseData);
                        }
                        else {
                            const body = this.getNodeParameter('fileContent', i);
                            if (fileName === '') {
                                throw new n8n_workflow_1.NodeOperationError(this.getNode(), 'File name must be set!', {
                                    itemIndex: i,
                                });
                            }
                            const encodedFilename = encodeURIComponent(fileName);
                            responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'PUT', `/drive/items/${parentId}:/${encodedFilename}:/content`, body, {}, undefined, { 'Content-Type': 'text/plain' });
                        }
                    }
                }
                if (resource === 'folder') {
                    if (operation === 'create') {
                        const names = this.getNodeParameter('name', i)
                            .split('/')
                            .filter((s) => s.trim() !== '');
                        const options = this.getNodeParameter('options', i);
                        let parentFolderId = options.parentFolderId ? options.parentFolderId : null;
                        for (const name of names) {
                            const body = {
                                name,
                                folder: {},
                            };
                            let endpoint = '/drive/root/children';
                            if (parentFolderId) {
                                endpoint = `/drive/items/${parentFolderId}/children`;
                            }
                            responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'POST', endpoint, body);
                            if (!responseData.id) {
                                break;
                            }
                            parentFolderId = responseData.id;
                        }
                    }
                    if (operation === 'delete') {
                        const folderId = this.getNodeParameter('folderId', i);
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'DELETE', `/drive/items/${folderId}`);
                        responseData = { success: true };
                    }
                    if (operation === 'getChildren') {
                        const folderId = this.getNodeParameter('folderId', i);
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/drive/items/${folderId}/children`);
                    }
                    if (operation === 'search') {
                        const query = this.getNodeParameter('query', i);
                        responseData = await GenericFunctions_1.microsoftApiRequestAllItems.call(this, 'value', 'GET', `/drive/root/search(q='${query}')`);
                        responseData = responseData.filter((item) => item.folder);
                    }
                    if (operation === 'share') {
                        const folderId = this.getNodeParameter('folderId', i);
                        const type = this.getNodeParameter('type', i);
                        const scope = this.getNodeParameter('scope', i);
                        const body = {
                            type,
                            scope,
                        };
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'POST', `/drive/items/${folderId}/createLink`, body);
                    }
                }
                if (resource === 'file' || resource === 'folder') {
                    if (operation === 'rename') {
                        const itemId = this.getNodeParameter('itemId', i);
                        const newName = this.getNodeParameter('newName', i);
                        const body = { name: newName };
                        responseData = await GenericFunctions_1.microsoftApiRequest.call(this, 'PATCH', `/drive/items/${itemId}`, body);
                    }
                }
            }
            catch (error) {
                if (this.continueOnFail()) {
                    if (resource === 'file' && operation === 'download') {
                        items[i].json = { error: error.message };
                    }
                    else {
                        const executionErrorData = this.helpers.constructExecutionMetaData(this.helpers.returnJsonArray({ error: error.message }), { itemData: { item: i } });
                        returnData.push(...executionErrorData);
                    }
                    continue;
                }
                throw error;
            }
            const executionData = this.helpers.constructExecutionMetaData(this.helpers.returnJsonArray(responseData), { itemData: { item: i } });
            returnData.push(...executionData);
        }
        if (resource === 'file' && operation === 'download') {
            return [items];
        }
        return [returnData];
    }
}
exports.MicrosoftOneDrive = MicrosoftOneDrive;
//# sourceMappingURL=MicrosoftOneDrive.node.js.map