"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MicrosoftSql = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const GenericFunctions_1 = require("./GenericFunctions");
const utilities_1 = require("../../../utils/utilities");
class MicrosoftSql {
    constructor() {
        this.description = {
            displayName: 'Microsoft SQL',
            name: 'microsoftSql',
            icon: 'file:mssql.svg',
            group: ['input'],
            version: 1,
            description: 'Get, add and update data in Microsoft SQL',
            defaults: {
                name: 'Microsoft SQL',
            },
            inputs: ['main'],
            outputs: ['main'],
            parameterPane: 'wide',
            credentials: [
                {
                    name: 'microsoftSql',
                    required: true,
                    testedBy: 'microsoftSqlConnectionTest',
                },
            ],
            properties: [
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    noDataExpression: true,
                    options: [
                        {
                            name: 'Execute Query',
                            value: 'executeQuery',
                            description: 'Execute an SQL query',
                            action: 'Execute a SQL query',
                        },
                        {
                            name: 'Insert',
                            value: 'insert',
                            description: 'Insert rows in database',
                            action: 'Insert rows in database',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update rows in database',
                            action: 'Update rows in database',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete rows in database',
                            action: 'Delete rows in database',
                        },
                    ],
                    default: 'insert',
                },
                {
                    displayName: 'Query',
                    name: 'query',
                    type: 'string',
                    noDataExpression: true,
                    typeOptions: {
                        editor: 'sqlEditor',
                        sqlDialect: 'MSSQL',
                    },
                    displayOptions: {
                        show: {
                            operation: ['executeQuery'],
                        },
                    },
                    default: '',
                    placeholder: 'SELECT id, name FROM product WHERE id < 40',
                    required: true,
                    description: 'The SQL query to execute',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: ['insert'],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to insert data to',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    requiresDataPath: 'multiple',
                    displayOptions: {
                        show: {
                            operation: ['insert'],
                        },
                    },
                    default: '',
                    placeholder: 'id,name,description',
                    description: 'Comma-separated list of the properties which should used as columns for the new rows',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: ['update'],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to update data in',
                },
                {
                    displayName: 'Update Key',
                    name: 'updateKey',
                    type: 'string',
                    requiresDataPath: 'single',
                    displayOptions: {
                        show: {
                            operation: ['update'],
                        },
                    },
                    default: 'id',
                    required: true,
                    description: 'Name of the property which decides which rows in the database should be updated. Normally that would be "id".',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    requiresDataPath: 'multiple',
                    displayOptions: {
                        show: {
                            operation: ['update'],
                        },
                    },
                    default: '',
                    placeholder: 'name,description',
                    description: 'Comma-separated list of the properties which should used as columns for rows to update',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: ['delete'],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to delete data',
                },
                {
                    displayName: 'Delete Key',
                    name: 'deleteKey',
                    type: 'string',
                    requiresDataPath: 'single',
                    displayOptions: {
                        show: {
                            operation: ['delete'],
                        },
                    },
                    default: 'id',
                    required: true,
                    description: 'Name of the property which decides which rows in the database should be deleted. Normally that would be "id".',
                },
            ],
        };
        this.methods = {
            credentialTest: {
                async microsoftSqlConnectionTest(credential) {
                    const credentials = credential.data;
                    try {
                        const pool = (0, GenericFunctions_1.configurePool)(credentials);
                        await pool.connect();
                    }
                    catch (error) {
                        return {
                            status: 'Error',
                            message: error.message,
                        };
                    }
                    return {
                        status: 'OK',
                        message: 'Connection successful!',
                    };
                },
            },
        };
    }
    async execute() {
        const credentials = await this.getCredentials('microsoftSql');
        const pool = (0, GenericFunctions_1.configurePool)(credentials);
        await pool.connect();
        let responseData = [];
        const items = this.getInputData();
        const operation = this.getNodeParameter('operation', 0);
        try {
            if (operation === 'executeQuery') {
                let rawQuery = this.getNodeParameter('query', 0);
                for (const resolvable of (0, utilities_1.getResolvables)(rawQuery)) {
                    rawQuery = rawQuery.replace(resolvable, this.evaluateExpression(resolvable, 0));
                }
                const { recordsets } = await pool.request().query(rawQuery);
                const result = recordsets.length > 1 ? (0, utilities_1.flatten)(recordsets) : recordsets[0];
                responseData = result;
            }
            else if (operation === 'insert') {
                const tables = (0, GenericFunctions_1.createTableStruct)(this.getNodeParameter, items);
                await (0, GenericFunctions_1.insertOperation)(tables, pool);
                responseData = items;
            }
            else if (operation === 'update') {
                const updateKeys = items.map((item, index) => this.getNodeParameter('updateKey', index));
                const tables = (0, GenericFunctions_1.createTableStruct)(this.getNodeParameter, items, ['updateKey'].concat(updateKeys), 'updateKey');
                await (0, GenericFunctions_1.updateOperation)(tables, pool);
                responseData = items;
            }
            else if (operation === 'delete') {
                const tables = items.reduce((acc, item, index) => {
                    const table = this.getNodeParameter('table', index);
                    const deleteKey = this.getNodeParameter('deleteKey', index);
                    if (acc[table] === undefined) {
                        acc[table] = {};
                    }
                    if (acc[table][deleteKey] === undefined) {
                        acc[table][deleteKey] = [];
                    }
                    acc[table][deleteKey].push(item);
                    return acc;
                }, {});
                responseData = await (0, GenericFunctions_1.deleteOperation)(tables, pool);
            }
            else {
                await pool.close();
                throw new n8n_workflow_1.NodeOperationError(this.getNode(), `The operation "${operation}" is not supported!`);
            }
        }
        catch (error) {
            if (this.continueOnFail()) {
                responseData = items;
            }
            else {
                await pool.close();
                throw error;
            }
        }
        await pool.close();
        const itemData = (0, utilities_1.generatePairedItemData)(items.length);
        const returnItems = this.helpers.constructExecutionMetaData(this.helpers.returnJsonArray(responseData), { itemData });
        return [returnItems];
    }
}
exports.MicrosoftSql = MicrosoftSql;
//# sourceMappingURL=MicrosoftSql.node.js.map