"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setValue = exports.getValue = exports.convertInfoToObject = exports.redisConnectionTest = exports.setupRedisClient = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const redis_1 = require("redis");
function setupRedisClient(credentials) {
    const redisOptions = {
        socket: {
            host: credentials.host,
            port: credentials.port,
        },
        database: credentials.database,
        password: credentials.password || undefined,
    };
    return (0, redis_1.createClient)(redisOptions);
}
exports.setupRedisClient = setupRedisClient;
async function redisConnectionTest(credential) {
    const credentials = credential.data;
    try {
        const client = setupRedisClient(credentials);
        await client.connect();
        await client.ping();
        return {
            status: 'OK',
            message: 'Connection successful!',
        };
    }
    catch (error) {
        return {
            status: 'Error',
            message: error.message,
        };
    }
}
exports.redisConnectionTest = redisConnectionTest;
function getParsedValue(value) {
    if (value.match(/^[\d\.]+$/) === null) {
        return value;
    }
    else {
        return parseFloat(value);
    }
}
function convertInfoToObject(stringData) {
    const returnData = {};
    let key, value;
    for (const line of stringData.split('\n')) {
        if (['#', ''].includes(line.charAt(0))) {
            continue;
        }
        [key, value] = line.split(':');
        if (key === undefined || value === undefined) {
            continue;
        }
        value = value.trim();
        if (value.includes('=')) {
            returnData[key] = {};
            let key2, value2;
            for (const keyValuePair of value.split(',')) {
                [key2, value2] = keyValuePair.split('=');
                returnData[key][key2] = getParsedValue(value2);
            }
        }
        else {
            returnData[key] = getParsedValue(value);
        }
    }
    return returnData;
}
exports.convertInfoToObject = convertInfoToObject;
async function getValue(client, keyName, type) {
    if (type === undefined || type === 'automatic') {
        type = await client.type(keyName);
    }
    if (type === 'string') {
        return await client.get(keyName);
    }
    else if (type === 'hash') {
        return await client.hGetAll(keyName);
    }
    else if (type === 'list') {
        return await client.lRange(keyName, 0, -1);
    }
    else if (type === 'sets') {
        return await client.sMembers(keyName);
    }
}
exports.getValue = getValue;
async function setValue(client, keyName, value, expire, ttl, type, valueIsJSON) {
    if (type === undefined || type === 'automatic') {
        if (typeof value === 'string') {
            type = 'string';
        }
        else if (Array.isArray(value)) {
            type = 'list';
        }
        else if (typeof value === 'object') {
            type = 'hash';
        }
        else {
            throw new n8n_workflow_1.NodeOperationError(this.getNode(), 'Could not identify the type to set. Please set it manually!');
        }
    }
    if (type === 'string') {
        await client.set(keyName, value.toString());
    }
    else if (type === 'hash') {
        if (valueIsJSON) {
            let values;
            if (typeof value === 'string') {
                try {
                    values = JSON.parse(value);
                }
                catch {
                    values = value;
                }
            }
            else {
                values = value;
            }
            for (const key of Object.keys(values)) {
                await client.hSet(keyName, key, values[key].toString());
            }
        }
        else {
            const values = value.toString().split(' ');
            await client.hSet(keyName, values);
        }
    }
    else if (type === 'list') {
        for (let index = 0; index < value.length; index++) {
            await client.lSet(keyName, index, value[index].toString());
        }
    }
    else if (type === 'sets') {
        await client.sAdd(keyName, value);
    }
    if (expire) {
        await client.expire(keyName, ttl);
    }
    return;
}
exports.setValue = setValue;
//# sourceMappingURL=utils.js.map