"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SwitchV3 = void 0;
const n8n_workflow_1 = require("n8n-workflow");
const set_1 = __importDefault(require("lodash/set"));
const utilities_1 = require("../../../utils/utilities");
const configuredOutputs = (parameters) => {
    var _a, _b, _c, _d;
    const mode = parameters.mode;
    if (mode === 'expression') {
        return Array.from({ length: parameters.numberOutputs }, (_, i) => ({
            type: `${"main"}`,
            displayName: i.toString(),
        }));
    }
    else {
        const rules = (_b = (_a = parameters.rules) === null || _a === void 0 ? void 0 : _a.values) !== null && _b !== void 0 ? _b : [];
        const ruleOutputs = rules.map((rule, index) => {
            return {
                type: `${"main"}`,
                displayName: rule.outputKey || index.toString(),
            };
        });
        if (((_c = parameters.options) === null || _c === void 0 ? void 0 : _c.fallbackOutput) === 'extra') {
            const renameFallbackOutput = (_d = parameters.options) === null || _d === void 0 ? void 0 : _d.renameFallbackOutput;
            ruleOutputs.push({
                type: `${"main"}`,
                displayName: renameFallbackOutput || 'Fallback',
            });
        }
        return ruleOutputs;
    }
};
class SwitchV3 {
    constructor(baseDescription) {
        this.methods = {
            loadOptions: {
                async getFallbackOutputOptions() {
                    var _a;
                    const rules = (_a = this.getCurrentNodeParameter('rules.values')) !== null && _a !== void 0 ? _a : [];
                    const outputOptions = [
                        {
                            name: 'None (default)',
                            value: 'none',
                            description: 'Items will be ignored',
                        },
                        {
                            name: 'Extra Output',
                            value: 'extra',
                            description: 'Items will be sent to the extra, separate, output',
                        },
                    ];
                    for (const [index, rule] of rules.entries()) {
                        outputOptions.push({
                            name: `Output ${rule.outputKey || index}`,
                            value: index,
                            description: `Items will be sent to the same output as when matched rule ${index + 1}`,
                        });
                    }
                    return outputOptions;
                },
            },
        };
        this.description = {
            ...baseDescription,
            subtitle: `=mode: {{(${utilities_1.capitalize})($parameter["mode"])}}`,
            version: [3],
            defaults: {
                name: 'Switch',
                color: '#506000',
            },
            inputs: ['main'],
            outputs: `={{(${configuredOutputs})($parameter)}}`,
            properties: [
                {
                    displayName: 'Mode',
                    name: 'mode',
                    type: 'options',
                    noDataExpression: true,
                    options: [
                        {
                            name: 'Rules',
                            value: 'rules',
                            description: 'Build a matching rule for each output',
                        },
                        {
                            name: 'Expression',
                            value: 'expression',
                            description: 'Write an expression to return the output index',
                        },
                    ],
                    default: 'rules',
                    description: 'How data should be routed',
                },
                {
                    displayName: 'Number of Outputs',
                    name: 'numberOutputs',
                    type: 'number',
                    displayOptions: {
                        show: {
                            mode: ['expression'],
                        },
                    },
                    default: 4,
                    description: 'How many outputs to create',
                },
                {
                    displayName: 'Output Index',
                    name: 'output',
                    type: 'number',
                    validateType: 'number',
                    hint: 'The index to route the item to, starts at 0',
                    displayOptions: {
                        show: {
                            mode: ['expression'],
                        },
                    },
                    default: '={{}}',
                    description: 'The output index to send the input item to. Use an expression to calculate which input item should be routed to which output. The expression must return a number.',
                },
                {
                    displayName: 'Routing Rules',
                    name: 'rules',
                    placeholder: 'Add Routing Rule',
                    type: 'fixedCollection',
                    typeOptions: {
                        multipleValues: true,
                        sortable: true,
                    },
                    default: {
                        values: [
                            {
                                conditions: {
                                    options: {
                                        caseSensitive: true,
                                        leftValue: '',
                                        typeValidation: 'strict',
                                    },
                                    conditions: [
                                        {
                                            leftValue: '',
                                            rightValue: '',
                                            operator: {
                                                type: 'string',
                                                operation: 'equals',
                                            },
                                        },
                                    ],
                                    combinator: 'and',
                                },
                            },
                        ],
                    },
                    displayOptions: {
                        show: {
                            mode: ['rules'],
                        },
                    },
                    options: [
                        {
                            name: 'values',
                            displayName: 'Values',
                            values: [
                                {
                                    displayName: 'Conditions',
                                    name: 'conditions',
                                    placeholder: 'Add Condition',
                                    type: 'filter',
                                    default: {},
                                    typeOptions: {
                                        multipleValues: false,
                                        filter: {
                                            caseSensitive: '={{!$parameter.options.ignoreCase}}',
                                            typeValidation: '={{$parameter.options.looseTypeValidation ? "loose" : "strict"}}',
                                        },
                                    },
                                },
                                {
                                    displayName: 'Rename Output',
                                    name: 'renameOutput',
                                    type: 'boolean',
                                    default: false,
                                },
                                {
                                    displayName: 'Output Name',
                                    name: 'outputKey',
                                    type: 'string',
                                    default: '',
                                    description: 'The label of output to which to send data to if rule matches',
                                    displayOptions: {
                                        show: {
                                            renameOutput: [true],
                                        },
                                    },
                                },
                            ],
                        },
                    ],
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    displayOptions: {
                        show: {
                            mode: ['rules'],
                        },
                    },
                    options: [
                        {
                            displayName: 'Fallback Output',
                            name: 'fallbackOutput',
                            type: 'options',
                            typeOptions: {
                                loadOptionsDependsOn: ['rules.values', '/rules', '/rules.values'],
                                loadOptionsMethod: 'getFallbackOutputOptions',
                            },
                            default: 'none',
                            description: 'If no rule matches the item will be sent to this output, by default they will be ignored',
                        },
                        {
                            displayName: 'Ignore Case',
                            description: 'Whether to ignore letter case when evaluating conditions',
                            name: 'ignoreCase',
                            type: 'boolean',
                            default: true,
                        },
                        {
                            displayName: 'Less Strict Type Validation',
                            description: 'Whether to try casting value types based on the selected operator',
                            name: 'looseTypeValidation',
                            type: 'boolean',
                            default: true,
                        },
                        {
                            displayName: 'Rename Fallback Output',
                            name: 'renameFallbackOutput',
                            type: 'string',
                            placeholder: 'e.g. Fallback',
                            default: '',
                            displayOptions: {
                                show: {
                                    fallbackOutput: ['extra'],
                                },
                            },
                        },
                        {
                            displayName: 'Send data to all matching outputs',
                            name: 'allMatchingOutputs',
                            type: 'boolean',
                            default: false,
                            description: 'Whether to send data to all outputs meeting conditions (and not just the first one)',
                        },
                    ],
                },
            ],
        };
    }
    async execute() {
        let returnData = [];
        const items = this.getInputData();
        const mode = this.getNodeParameter('mode', 0);
        const checkIndexRange = (returnDataLength, index, itemIndex = 0) => {
            if (Number(index) === returnDataLength) {
                throw new n8n_workflow_1.NodeOperationError(this.getNode(), `The ouput ${index} is not allowed. `, {
                    itemIndex,
                    description: `Output indexes are zero based, if you want to use the extra output use ${index - 1}`,
                });
            }
            if (index < 0 || index > returnDataLength) {
                throw new n8n_workflow_1.NodeOperationError(this.getNode(), `The ouput ${index} is not allowed`, {
                    itemIndex,
                    description: `It has to be between 0 and ${returnDataLength - 1}`,
                });
            }
        };
        itemLoop: for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
            try {
                const item = items[itemIndex];
                item.pairedItem = { item: itemIndex };
                if (mode === 'expression') {
                    const numberOutputs = this.getNodeParameter('numberOutputs', itemIndex);
                    if (itemIndex === 0) {
                        returnData = new Array(numberOutputs).fill(0).map(() => []);
                    }
                    const outputIndex = this.getNodeParameter('output', itemIndex);
                    checkIndexRange(returnData.length, outputIndex, itemIndex);
                    returnData[outputIndex].push(item);
                }
                else if (mode === 'rules') {
                    const rules = this.getNodeParameter('rules.values', itemIndex, []);
                    if (!rules.length)
                        continue;
                    const options = this.getNodeParameter('options', itemIndex, {});
                    const fallbackOutput = options.fallbackOutput;
                    if (itemIndex === 0) {
                        returnData = new Array(rules.length).fill(0).map(() => []);
                        if (fallbackOutput === 'extra') {
                            returnData.push([]);
                        }
                    }
                    let matchFound = false;
                    for (const [ruleIndex, rule] of rules.entries()) {
                        let conditionPass;
                        try {
                            conditionPass = this.getNodeParameter(`rules.values[${ruleIndex}].conditions`, itemIndex, false, {
                                extractValue: true,
                            });
                        }
                        catch (error) {
                            if (!options.looseTypeValidation) {
                                error.description =
                                    "Try changing the type of comparison. Alternatively you can enable 'Less Strict Type Validation' in the options.";
                            }
                            (0, set_1.default)(error, 'context.itemIndex', itemIndex);
                            (0, set_1.default)(error, 'node', this.getNode());
                            throw error;
                        }
                        if (conditionPass) {
                            matchFound = true;
                            checkIndexRange(returnData.length, rule.output, itemIndex);
                            returnData[ruleIndex].push(item);
                            if (!options.allMatchingOutputs) {
                                continue itemLoop;
                            }
                        }
                    }
                    if (fallbackOutput !== undefined && fallbackOutput !== 'none' && !matchFound) {
                        if (fallbackOutput === 'extra') {
                            returnData[returnData.length - 1].push(item);
                            continue;
                        }
                        checkIndexRange(returnData.length, fallbackOutput, itemIndex);
                        returnData[fallbackOutput].push(item);
                    }
                }
            }
            catch (error) {
                if (this.continueOnFail()) {
                    returnData[0].push({ json: { error: error.message } });
                    continue;
                }
                throw new n8n_workflow_1.NodeOperationError(this.getNode(), error);
            }
        }
        if (!returnData.length)
            return [[]];
        return returnData;
    }
}
exports.SwitchV3 = SwitchV3;
//# sourceMappingURL=SwitchV3.node.js.map